#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Date   :2024/11/11 13:57
# @Author : liangchunhua
# @Desc   :
import os

from pastor.models import StepTypeEnum


def is_headless():
    return os.getenv("HEADLESS", False)


def get_page_load_timeout():
    return os.getenv("PAGE_LOAD_TIMEOUT", 30)


def get_implicitly_wait_time():
    return os.getenv("IMPLICITLY_WAIT_TIME", 3)


def get_window_size():
    return os.getenv("WINDOW_SIZE", '--start-maximized')


def get_remote_url():
    return os.getenv("REMOTE_URL", 'http://127.0.0.1:4444/wb/hub')


def get_domain():
    # 读取cookie数据时的domain,统一登录平台和实际应用url不一致，需要替换
    return os.getenv("DOMAIN", '.lyky.com.cn')


def is_read_cache():
    return os.getenv("CACHE", 'false').lower()


def get_browser():
    from pastor.uicore.driver import BrowserTypeEnum
    return os.getenv("BROWSER", BrowserTypeEnum.CHROME)


def gh_token():
    # gitlab接口请求token
    return os.getenv("GH_TOKEN", None)


def step_type():
    # 执行类型，api或ui
    return os.getenv('TYPE', StepTypeEnum.API).upper()


def database_url():
    # 默认database配置: mysql+pymysql://{username}:{password}@{host}:{port}/{database}?charset=utf8mb4
    return os.getenv('DATABASE_URL', None)


def driver_latest_release_url():
    # 获取webdriver最后版本
    return os.getenv('DRIVER_LATEST_RELEASE_URL', None)


def driver_download_url():
    # 获取webdriver驱动下载信息
    return os.getenv('DRIVER_DOWNLOAD_URL', None)

def base_url():
    # 获取webdriver驱动下载信息
    return os.getenv('BASE_URL', None)



import uuid
from typing import List

import pytest
from pastor import Config, Step
from loguru import logger


@pytest.fixture(scope="session", autouse=True)
def session_fixture(request):
    """setup and teardown each task"""
    total_testcases_num = request.node.testscollected
    testcases = []
    for item in request.node.items:
        testcase = {
            "name": item.cls.config.name,
            "path": item.cls.config.path,
            "node_id": item.nodeid,
        }
        testcases.append(testcase)

    logger.debug(f"collected {total_testcases_num} testcases: {testcases}")

    yield

    logger.debug(f"teardown task fixture")

    # teardown task
    # TODO: upload task summary


@pytest.fixture(scope="function", autouse=True)
def testcase_fixture(request):
    """setup and teardown each testcase"""
    config: Config = request.cls.config
    teststeps: List[Step] = request.cls.teststeps

    logger.debug(f"setup testcase fixture: {config.name} - {request.module.__name__}")

    def update_request_headers(steps, index):
        for teststep in steps:
            if teststep.request:
                index += 1
                teststep.request.headers["X-Request-ID"] = f"{prefix}-{index}"
            elif teststep.testcase and hasattr(teststep.testcase, "teststeps"):
                update_request_headers(teststep.testcase.teststeps, index)

    # you can update testcase teststep like this
    prefix = f"HRUN-{uuid.uuid4()}"
    update_request_headers(teststeps, 0)

    yield

    logger.debug(
        f"teardown testcase fixture: {config.name} - {request.module.__name__}"
    )

    summary = request.instance.get_summary()
    logger.debug(f"testcase result summary: {summary}")

    # TODO: upload testcase summary



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Date   :2024/11/6 9:21
# @Author : liangchunhua
# @Desc   :
from typing import List, Any, Text

from selenium.webdriver.remote.webelement import WebElement


def check_eq(elements: List[WebElement], exp: Any, message: Text = ""):
    assert True


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @time   :2024/10/29 14:14
# @Author : liangchunhua
# @Desc   : 数据库引擎
import datetime
import json
import time

from sqlalchemy import create_engine, text, Row
from sqlalchemy.orm import sessionmaker, DeclarativeMeta


class DBEngine(object):
    def __init__(self, db_uri: str):
        """
        db_uri = f'mysql+pymysql://{username}:{password}@{host}:{port}/{database}?charset=utf8mb4'
        """
        engine = create_engine(db_uri)
        session = sessionmaker(bind=engine)
        from sqlalchemy.orm import scoped_session
        # 多线程
        scoped_session = scoped_session(session)
        self.__session = scoped_session

    @staticmethod
    def __value_decode(row: dict):
        """
        Try to decode value of table
        datetime.datetime-->string
        datetime.date-->string
        json str-->dict
        :param row:
        :return:
        """
        for k, v in row.items():
            if isinstance(v, datetime.datetime):
                row[k] = v.strftime("%Y-%m-%d %H:%M:%S")
            elif isinstance(v, datetime.date):
                row[k] = v.strftime("%Y-%m-%d")
            elif isinstance(v, str):
                try:
                    row[k] = json.loads(v)
                except ValueError:
                    pass

    def __fetch(self, query: str, size: int = -1, commit: bool = True):
        query = query.strip()
        result = self.__session.execute(text(query))
        self.__session.commit()
        if query.upper()[:6] == "SELECT":
            if size < 0:
                # al = result.fetchall()
                # from itertools import chain
                # al = list(chain.from_iterable(al))
                # al = DBEngine.__default_serialize(al)
                mny = result.fetchall()
                mny = [row._asdict() for row in mny]
                return mny or None
            else:
                mny = result.fetchmany(size)
                mny = [row._asdict() for row in mny]
                # for el in mny:
                #     self.value_decode(el)
                return mny or None
        elif query.upper()[:6] in ("UPDATE", "DELETE", "INSERT"):
            return {"count": result.rowcount}

    @staticmethod
    def __default_serialize(obj):
        """默认序序列化"""
        try:
            if isinstance(obj, datetime.datetime):
                return obj.strftime("%Y-%m-%d %H:%M:%S")
            if isinstance(obj, Row):
                return {key: DBEngine.__default_serialize(value) for key, value in
                        dict(zip(obj._fields, obj._data)).items()}
            if isinstance(obj, list):
                return [DBEngine.__default_serialize(e) for e in obj]
            if hasattr(obj, "__class__") and isinstance(obj.__class__, DeclarativeMeta):
                return {c.name: DBEngine.__default_serialize(getattr(obj, c.name)) for c in obj.__table__.columns}
            return obj
        except TypeError as err:
            return repr(obj)

    def fetchone(self, query, commit=True):
        return self.__fetch(query, size=1, commit=commit)

    def fetchmany(self, query, size=1000, commit=True):
        return self.__fetch(query=query, size=size, commit=commit)

    def fetchall(self, query, commit=True):
        return self.__fetch(query=query, size=-1, commit=commit)

    def insert(self, query, commit=True):
        return self.__fetch(query=query, commit=commit)

    def delete(self, query, commit=True):
        return self.__fetch(query=query, commit=commit)

    def update(self, query, commit=True):
        return self.__fetch(query=query, commit=commit)

    def close(self):
        if self.__session:
            self.__session.close()

    def __is_timestamp(value):
        try:
            # 尝试将数据转换为float类型
            timestamp = float(value)
            # 检查时间戳是否在可接受的范围内
            return timestamp > 0 and timestamp < time.time() + 253402300800  # 2106年的时间戳
        except (ValueError, TypeError):
            # 转换失败，不是时间戳
            return False

if __name__ == "__main__":
    user = 'root'
    password = 'rJCuKfeVB6nUZfSk'
    host = '47.101.134.204'
    port = '3306'
    database = 'lykytestplatform'
    # db = DBEngine(f'mysql+pymysql://{user}:'
    #               f'{password}@{host}:'
    #               f'{port}/{database}'
    #               f"?charset=utf8mb4")
    db = DBEngine(f'mysql+pymysql://root:rJCuKfeVB6nUZfSk@47.101.134.204:3306/sit_cas_new?charset=utf8mb4')
    result = db.fetchone('select code from cn_sms_code order by created_at desc limit 1')
    print(result)



"""
脚手架
"""

import os.path
import subprocess
import sys

from loguru import logger

from pastor.utils import ga_client


def init_parser_scaffold(subparsers):
    sub_parser_scaffold = subparsers.add_parser(
        "startproject", help="Create a new project with template structure."
    )
    sub_parser_scaffold.add_argument(
        "project_name", type=str, nargs="?", help="Specify new project name."
    )
    return sub_parser_scaffold


def create_scaffold(project_name):
    """ create scaffold with specified project name.
    """

    def show_tree(prj_name):
        try:
            print(f"\n$ tree {prj_name} -a")
            subprocess.run(["tree", prj_name, "-a"])
            print("")
        except OSError:
            logger.warning("tree command not exists, ignore.")

    if os.path.isdir(project_name):
        logger.warning(
            f"Project folder {project_name} exists, please specify a new project name."
        )
        show_tree(project_name)
        return 1
    elif os.path.isfile(project_name):
        logger.warning(
            f"Project name {project_name} conflicts with existed file, please specify a new one."
        )
        return 1

    logger.info(f"Create new project: {project_name}")
    print(f"Project Root Dir: {os.path.join(os.getcwd(), project_name)}\n")

    def create_folder(path):
        os.makedirs(path)
        msg = f"created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"created file: {path}"
        print(msg)

    demo_testcase_request_content = """
config:
    name: "request methods testcase with functions"
    variables:
        foo1: config_bar1
        foo2: config_bar2
        expect_foo1: config_bar1
        expect_foo2: config_bar2
    base_url: "https://postman-echo.com"
    verify: False
    export: ["foo3"]

teststeps:
-
    name: get with params
    variables:
        foo1: bar11
        foo2: bar21
        sum_v: "${sum_two(1, 2)}"
    request:
        method: GET
        url: /get
        params:
            foo1: $foo1
            foo2: $foo2
            sum_v: $sum_v
        headers:
            User-Agent: pastor/${get_pastor_version()}
    extract:
        foo3: "body.args.foo2"
    validate:
        - eq: ["status_code", 200]
        - eq: ["body.args.foo1", "bar11"]
        - eq: ["body.args.sum_v", "3"]
        - eq: ["body.args.foo2", "bar21"]
-
    name: post raw text
    variables:
        foo1: "bar12"
        foo3: "bar32"
    request:
        method: POST
        url: /post
        headers:
            User-Agent: pastor/${get_pastor_version()}
            Content-Type: "text/plain"
        data: "This is expected to be sent back as part of response body: $foo1-$foo2-$foo3."
    validate:
        - eq: ["status_code", 200]
        - eq: ["body.data", "This is expected to be sent back as part of response body: bar12-$expect_foo2-bar32."]
-
    name: post form data
    variables:
        foo2: bar23
    request:
        method: POST
        url: /post
        headers:
            User-Agent: pastor/${get_pastor_version()}
            Content-Type: "application/x-www-form-urlencoded"
        data: "foo1=$foo1&foo2=$foo2&foo3=$foo3"
    validate:
        - eq: ["status_code", 200]
        - eq: ["body.form.foo1", "$expect_foo1"]
        - eq: ["body.form.foo2", "bar23"]
        - eq: ["body.form.foo3", "bar21"]
"""
    demo_testcase_with_ref_content = """
config:
    name: "request methods testcase: reference testcase"
    variables:
        foo1: testsuite_config_bar1
        expect_foo1: testsuite_config_bar1
        expect_foo2: config_bar2
    base_url: "https://postman-echo.com"
    verify: False

teststeps:
-
    name: request with functions
    variables:
        foo1: testcase_ref_bar1
        expect_foo1: testcase_ref_bar1
    testcase: testcases/demo_testcase_request.yml
    export:
        - foo3
-
    name: post form data
    variables:
        foo1: bar1
    request:
        method: POST
        url: /post
        headers:
            User-Agent: pastor/${get_pastor_version()}
            Content-Type: "application/x-www-form-urlencoded"
        data: "foo1=$foo1&foo2=$foo3"
    validate:
        - eq: ["status_code", 200]
        - eq: ["body.form.foo1", "bar1"]
        - eq: ["body.form.foo2", "bar21"]
"""
    ignore_content = "\n".join(
        [".env", "reports/*", "__pycache__/*", "*.pyc", ".python-version", "logs/*"]
    )
    demo_debugtalk_content = """import time

from pastor import __version__


def get_pastor_version():
    return __version__


def sum_two(m, n):
    return m + n


def sleep(n_secs):
    time.sleep(n_secs)
"""
    demo_env_content = "\n".join(["TYPE=ui", "GH_TOKEN=glpat-Bq4A8tobo1wzKm9kzuN7"])

    demo_testcase_with_ui = """
config:
    name: "request methods testcase with functions"
    variables:
        value: "#div-hover .dropbtn"
        expect_foo1: '123'
    ignore_cache: true

teststeps:
-
    name: 输入账号密码
    variables:
        send_keys: send_keys
    location:
        - by: ID
          value: input
          action: $send_keys
          data: 'b2cnewtester3'
          desc: '输入用户名'
        - by: ID
          value: 'normal_login_password'
          action: $send_keys
          data: 'aa123456'
          desc: '输入密码'
        - by: CSS_SELECTOR
          value: ".login-agreement .ant-checkbox-input"
          action: click
          sleep: 1
          desc: '勾选同意'
        - by: CSS_SELECTOR
          value: ".login-button-box .login-button"
          action: click
          desc: '点击登录'
    validate:
      - eq:
          by: ID
          value: "phone"
          disabled: 'true'
      - eq:
          by: XPATH
          value: "//input[@autocomplete='one-time-code']"
          placeholder: '输入你的验证码'
      - eq:
          by: XPATH
          value: "//input[@autocomplete='one-time-code']"
          type: 'text'
      - eq:
          by: CSS_SELECTOR
          value: ".ant-input-group-addon button"
          val: '发送验证码'
      - eq:
          by: CSS_SELECTOR
          value: ".ant-form-item-control-input-content button[type='submit']"
          val: '下一步'
-
    name: 发送验证码
    location:
        - by: CSS_SELECTOR
          value: ".ant-input-group-addon button"
          action: click
          desc: '发送验证码'
          sleep: 2
    validate:
      - contains:
          by: CSS_SELECTOR
          value: ".code-input button"
          disabled: 'true'

-
    name: 输入验证码登录
    pre_sql:
      - datasource: test
        dml:
          - 'select code from cn_sms_code order by created_at desc limit 1'
    location:
        - by: XPATH
          value: "//input[@autocomplete='one-time-code']"
          action: send_keys
          data: $code
          desc: '输入正确验证码'
        - by: CSS_SELECTOR
          value: ".ant-form-item-control-input-content button[type='submit']"
          action: click
          desc: '点击下一步'
          sleep: 2
        - action: visibility_of_element_located
          data: '15'
          by: XPATH
          value: "//span[@class='i-layout-menu-side-title-text']"
          desc: '等待首页元素'
    post_sql:
      - datasource: test
        dml:
          - 'select code as code2 from cn_sms_code order by created_at desc limit 1'
    validate:
      - contains:
          by: XPATH
          value: "//span[@class='i-layout-menu-side-title-text']"
          val: '商家首页'
    """

    create_folder(project_name)
    create_folder(os.path.join(project_name, "har"))
    create_folder(os.path.join(project_name, "testcases"))
    create_folder(os.path.join(project_name, "reports"))

    create_file(
        os.path.join(project_name, "testcases", "demo_testcase_request.yml"),
        demo_testcase_request_content,
    )
    create_file(
        os.path.join(project_name, "testcases", "demo_testcase_ref.yml"),
        demo_testcase_with_ref_content,
    )
    create_file(
        os.path.join(project_name, "testcases", "login.yml"),
        demo_testcase_with_ui,
    )
    create_file(os.path.join(project_name, "debugtalk.py"), demo_debugtalk_content)
    create_file(os.path.join(project_name, ".env"), demo_env_content)
    create_file(os.path.join(project_name, ".gitignore"), ignore_content)

    show_tree(project_name)
    return 0


def main_scaffold(args):
    ga_client.track_event("Scaffold", "startproject")
    sys.exit(create_scaffold(args.project_name))



import typing
from contextvars import ContextVar


class Local:
    __slots__ = ("_storage",)

    def __init__(self) -> None:
        object.__setattr__(self, "_storage", ContextVar("local_storage"))

    def __iter__(self) -> typing.Iterator[typing.Tuple[int, typing.Any]]:
        return iter(self._storage.get({}).items())

    def __release_local__(self) -> None:
        self._storage.set({})

    def __getattr__(self, name: str) -> typing.Any:
        values = self._storage.get({})
        try:
            return values[name]
        except KeyError:
            return None

    def __setattr__(self, name: str, value: typing.Any) -> None:
        values = self._storage.get({}).copy()
        values[name] = value
        self._storage.set(values)

    def __delattr__(self, name: str) -> None:
        values = self._storage.get({}).copy()
        try:
            del values[name]
            self._storage.set(values)
        except KeyError:
            ...


gc = Local()



import os
import random
import string
import time
import uuid

from loguru import logger


def get_httpbin_server():
    return "https://httpbin.org"


def setup_testcase(variables):
    logger.info(f"setup_testcase, variables: {variables}")
    variables["request_id_prefix"] = str(int(time.time()))


def teardown_testcase():
    logger.info(f"teardown_testcase.")


def setup_teststep(request, variables):
    logger.info(f"setup_teststep, request: {request}, variables: {variables}")
    request.setdefault("headers", {})
    request_id_prefix = variables["request_id_prefix"]
    request["headers"]["HRUN-Request-ID"] = request_id_prefix + "-" + str(uuid.uuid4())


def teardown_teststep(response):
    logger.info(f"teardown_teststep, response status code: {response.status_code}")


def sum_two(m, n):
    return m + n


def sum_status_code(status_code, expect_sum):
    """ sum status code digits
        e.g. 400 => 4, 201 => 3
    """
    sum_value = 0
    for digit in str(status_code):
        sum_value += int(digit)

    assert sum_value == expect_sum


def is_status_code_200(status_code):
    return status_code == 200


os.environ["TEST_ENV"] = "PRODUCTION"


def skip_test_in_production_env():
    """ skip this test in production environment
    """
    return os.environ["TEST_ENV"] == "PRODUCTION"


def get_user_agent():
    return ["iOS/10.1", "iOS/10.2"]


def gen_app_version():
    return [{"app_version": "2.8.5"}, {"app_version": "2.8.6"}]


def get_account():
    return [
        {"username": "user1", "password": "111111"},
        {"username": "user2", "password": "222222"},
    ]


def get_account_in_tuple():
    return [("user1", "111111"), ("user2", "222222")]


def gen_random_string(str_len):
    random_char_list = []
    for _ in range(str_len):
        random_char = random.choice(string.ascii_letters + string.digits)
        random_char_list.append(random_char)

    random_string = "".join(random_char_list)
    return random_string


def setup_hook_add_kwargs(request):
    request["key"] = "value"


def setup_hook_remove_kwargs(request):
    request.pop("key")


def teardown_hook_sleep_N_secs(response, n_secs):
    """ sleep n seconds after request
    """
    if response.status_code == 200:
        time.sleep(0.1)
    else:
        time.sleep(n_secs)


def hook_print(msg):
    print(msg)


def modify_request_json(request, os_platform):
    request["json"]["os_platform"] = os_platform


def setup_hook_httpntlmauth(request):
    if "httpntlmauth" in request:
        from requests_ntlm import HttpNtlmAuth

        auth_account = request.pop("httpntlmauth")
        request["auth"] = HttpNtlmAuth(
            auth_account["username"], auth_account["password"]
        )


def alter_response(response):
    response.status_code = 500
    response.headers["Content-Type"] = "html/text"
    response.body["headers"]["Host"] = "127.0.0.1:8888"
    response.new_attribute = "new_attribute_value"
    response.new_attribute_dict = {"key": 123}


def alter_response_302(response):
    response.status_code = 500
    response.headers["Content-Type"] = "html/text"
    response.text = "abcdef"
    response.new_attribute = "new_attribute_value"
    response.new_attribute_dict = {"key": 123}


def alter_response_error(response):
    # NameError
    not_defined_variable


def gen_variables():
    return {"var_a": 1, "var_b": 2}



import collections
import copy
import itertools
import json
import os
import os.path
import platform
import uuid
from multiprocessing import Queue
from typing import Any, Dict, List, Text

import requests
import sentry_sdk
from loguru import logger

from pastor import __version__, exceptions
from pastor.models import VariablesMapping


def init_sentry_sdk():
    sentry_sdk.init(
        dsn="https://460e31339bcb428c879aafa6a2e78098@sentry.io/5263855",
        release="pastor@{}".format(__version__),
    )
    with sentry_sdk.configure_scope() as scope:
        scope.set_user({"id": uuid.getnode()})


class GAClient(object):

    version = '1'   # GA API Version
    report_url = 'https://www.google-analytics.com/collect'
    report_debug_url = 'https://www.google-analytics.com/debug/collect'   # used for debug

    def __init__(self, tracking_id: Text):
        self.http_client = requests.Session()
        self.label = f"v{__version__}"
        self.common_params = {
            'v': self.version,
            'tid': tracking_id,    # Tracking ID / Property ID, XX-XXXXXXX-X
            'cid': uuid.getnode(),      # Anonymous Client ID
            'ua': f'Pastor/{__version__}',
        }
        # do not send GA events in CI environment
        self.__is_ci = os.getenv("DISABLE_GA") == "true"

    def track_event(self, category: Text, action: Text, value: int = 0):
        if self.__is_ci:
            return

        data = {
            't': 'event',       # Event hit type = event
            'ec': category,     # Required. Event Category.
            'ea': action,       # Required. Event Action.
            'el': self.label,   # Optional. Event label, used as version.
            'ev': value,        # Optional. Event value, must be non-negative integer
        }
        data.update(self.common_params)
        try:
            self.http_client.post(self.report_url, data=data, timeout=5)
        except Exception:   # ProxyError, SSLError, ConnectionError
            pass

    def track_user_timing(self, category: Text, variable: Text, duration: int):
        if self.__is_ci:
            return

        data = {
            't': 'timing',      # Event hit type = timing
            'utc': category,    # Required. user timing category. e.g. jsonLoader
            'utv': variable,    # Required. timing variable. e.g. load
            'utt': duration,    # Required. time took duration.
            'utl': self.label,  # Optional. user timing label, used as version.
        }
        data.update(self.common_params)
        try:
            self.http_client.post(self.report_url, data=data, timeout=5)
        except Exception:   # ProxyError, SSLError, ConnectionError
            pass


ga_client = GAClient("UA-114587036-1")


def set_os_environ(variables_mapping):
    """ set variables mapping to os.environ
    """
    for variable in variables_mapping:
        os.environ[variable] = variables_mapping[variable]
        logger.debug(f"Set OS environment variable: {variable}")


def unset_os_environ(variables_mapping):
    """ unset variables mapping to os.environ
    """
    for variable in variables_mapping:
        os.environ.pop(variable)
        logger.debug(f"Unset OS environment variable: {variable}")


def get_os_environ(variable_name):
    """ get value of environment variable.

    Args:
        variable_name(str): variable name

    Returns:
        value of environment variable.

    Raises:
        exceptions.EnvNotFound: If environment variable not found.

    """
    try:
        return os.environ[variable_name]
    except KeyError:
        raise exceptions.EnvNotFound(variable_name)


def lower_dict_keys(origin_dict):
    """ convert keys in dict to lower case

    Args:
        origin_dict (dict): mapping data structure

    Returns:
        dict: mapping with all keys lowered.

    Examples:
        >>> origin_dict = {
            "Name": "",
            "Request": "",
            "URL": "",
            "METHOD": "",
            "Headers": "",
            "Data": ""
        }
        >>> lower_dict_keys(origin_dict)
            {
                "name": "",
                "request": "",
                "url": "",
                "method": "",
                "headers": "",
                "data": ""
            }

    """
    if not origin_dict or not isinstance(origin_dict, dict):
        return origin_dict

    return {key.lower(): value for key, value in origin_dict.items()}


def print_info(info_mapping):
    """ print info in mapping.

    Args:
        info_mapping (dict): input(variables) or output mapping.

    Examples:
        >>> info_mapping = {
                "var_a": "hello",
                "var_b": "world"
            }
        >>> info_mapping = {
                "status_code": 500
            }
        >>> print_info(info_mapping)
        ==================== Output ====================
        Key              :  Value
        ---------------- :  ----------------------------
        var_a            :  hello
        var_b            :  world
        ------------------------------------------------

    """
    if not info_mapping:
        return

    content_format = "{:<16} : {:<}\n"
    content = "\n==================== Output ====================\n"
    content += content_format.format("Variable", "Value")
    content += content_format.format("-" * 16, "-" * 29)

    for key, value in info_mapping.items():
        if isinstance(value, (tuple, collections.deque)):
            continue
        elif isinstance(value, (dict, list)):
            value = json.dumps(value)
        elif value is None:
            value = "None"

        content += content_format.format(key, value)

    content += "-" * 48 + "\n"
    logger.info(content)


def omit_long_data(body, omit_len=512):
    """ omit too long str/bytes
    """
    if not isinstance(body, (str, bytes)):
        return body

    body_len = len(body)
    if body_len <= omit_len:
        return body

    omitted_body = body[0:omit_len]

    appendix_str = f" ... OMITTED {body_len - omit_len} CHARACTORS ..."
    if isinstance(body, bytes):
        appendix_str = appendix_str.encode("utf-8")

    return omitted_body + appendix_str


def get_platform():
    return {
        "pastor_version": __version__,
        "python_version": "{} {}".format(
            platform.python_implementation(), platform.python_version()
        ),
        "platform": platform.platform(),
    }


def sort_dict_by_custom_order(raw_dict: Dict, custom_order: List):
    def get_index_from_list(lst: List, item: Any):
        try:
            return lst.index(item)
        except ValueError:
            # item is not in lst
            return len(lst) + 1

    return dict(
        sorted(raw_dict.items(), key=lambda i: get_index_from_list(custom_order, i[0]))
    )


class ExtendJSONEncoder(json.JSONEncoder):
    """ especially used to safely dump json data with python object, such as MultipartEncoder
    """

    def default(self, obj):
        try:
            return super(ExtendJSONEncoder, self).default(obj)
        except (UnicodeDecodeError, TypeError):
            return repr(obj)


def merge_variables(
    variables: VariablesMapping, variables_to_be_overridden: VariablesMapping
) -> VariablesMapping:
    """ merge two variables mapping, the first variables have higher priority
    """
    step_new_variables = {}
    for key, value in variables.items():
        if f"${key}" == value or "${" + key + "}" == value:
            # e.g. {"base_url": "$base_url"}
            # or {"base_url": "${base_url}"}
            continue

        step_new_variables[key] = value

    merged_variables = copy.copy(variables_to_be_overridden)
    merged_variables.update(step_new_variables)
    return merged_variables


def is_support_multiprocessing() -> bool:
    try:
        Queue()
        return True
    except (ImportError, OSError):
        # system that does not support semaphores(dependency of multiprocessing), like Android termux
        return False


def gen_cartesian_product(*args: List[Dict]) -> List[Dict]:
    """ generate cartesian product for lists

    Args:
        args (list of list): lists to be generated with cartesian product

    Returns:
        list: cartesian product in list

    Examples:

        >>> arg1 = [{"a": 1}, {"a": 2}]
        >>> arg2 = [{"x": 111, "y": 112}, {"x": 121, "y": 122}]
        >>> args = [arg1, arg2]
        >>> gen_cartesian_product(*args)
        >>> # same as below
        >>> gen_cartesian_product(arg1, arg2)
            [
                {'a': 1, 'x': 111, 'y': 112},
                {'a': 1, 'x': 121, 'y': 122},
                {'a': 2, 'x': 111, 'y': 112},
                {'a': 2, 'x': 121, 'y': 122}
            ]

    """
    if not args:
        return []
    elif len(args) == 1:
        return args[0]

    product_list = []
    for product_item_tuple in itertools.product(*args):
        product_item_dict = {}
        for item in product_item_tuple:
            product_item_dict.update(item)

        product_list.append(product_item_dict)

    return product_list



import os
import typing
from enum import Enum
from typing import Any
from typing import Dict, Text, Union, Callable
from typing import List

from pydantic import BaseModel, Field, field_validator, ValidationError
from pydantic import HttpUrl

Name = Text
Url = Text
BaseUrl = Union[HttpUrl, Text]
VariablesMapping = Dict[Text, Any]
FunctionsMapping = Dict[Text, Callable]
Headers = Dict[Text, Text]
Cookies = Dict[Text, Text]
Verify = bool
Hooks = List[Union[Text, Dict[Text, Text]]]
Export = List[Text]
Validators = List[Dict]
Env = Dict[Text, Any]


class MethodEnum(Text, Enum):
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    DELETE = "DELETE"
    HEAD = "HEAD"
    OPTIONS = "OPTIONS"
    PATCH = "PATCH"


class StepTypeEnum(str, Enum):
    """步骤类型枚举"""
    API = "API"
    SQL = "SQL"
    UI = "UI"
    UNKNOWN = ''


class TConfig(BaseModel):
    name: Name
    verify: Verify = False
    base_url: BaseUrl = ""
    # Text: prepare variables in debugtalk.py, ${gen_variables()}
    variables: Union[VariablesMapping, Text] = {}
    parameters: Union[VariablesMapping, Text] = {}
    # setup_hooks: Hooks = []
    # teardown_hooks: Hooks = []
    export: Export = []
    path: Text = None
    weight: int = 1
    datasource: str = None
    # 忽略浏览器缓存
    ignore_cache: bool = False


class TRequest(BaseModel):
    """requests.Request model"""

    method: MethodEnum
    url: Url
    params: Dict[Text, Text] = {}
    headers: Headers = {}
    req_json: Union[Dict, List, Text] = Field(None, alias="json")
    data: Union[Text, Dict[Text, Any]] = None
    cookies: Cookies = {}
    timeout: float = 120
    allow_redirects: bool = True
    verify: Verify = False
    upload: Dict = {}  # used for upload files


class TUiLocation(BaseModel):
    """ui模型"""
    action: str = Field(None, description="动作")
    data: Union[Dict, List, Text, int] = Field({}, description="输入数据")
    by: typing.Literal[
        'ID', 'XPATH', 'LINK_TEXT', 'PARTIAL_LINK_TEXT', 'NAME', 'TAG_NAME', 'CLASS_NAME', 'CSS_SELECTOR'] = Field(None,
                                                                                                                   description="定位器")
    value: str = Field(None, description="定位值")
    sleep: int = Field(0, ge=0, description="睡眠时间")
    desc: str = Field(None, description="描述")
    time: int = Field(0, ge=0, description="等待元素时间")
    skip: Union[bool, Text] = Field(False, description="是否跳过")
    # @field_validator('action')
    # def check_action_func_name(cls,v):
    #     """
    #     校验action的传参,需要匹配Action的静态方法
    #     :param v:
    #     :return:
    #     """
    #     from pastor.uicore.web_action import Action
    #     if '$' in v or v is None:
    #         return v
    #     funcs = [w for w in dir(Action) if callable(getattr(Action, w)) and not w.startswith("__")]
    #     if v not in funcs:
    #         raise ValueError(f'action: {v}, 操作方法不存在以下列表中:{funcs}')
    #     return v


class SqlData(BaseModel):
    """sql数据"""
    datasource: str = Field(None, description="数据源")
    dml: List[str] = Field([], description="sql语句")

    @field_validator('dml')
    def check_dml_content(cls, v):
        for i in v:
            if '*' in i:
                raise ValidationError(f'{i},不能使用*查询,请查询具体字段！')
        return v


class TStep(BaseModel):
    name: Name
    request: Union[TRequest, None] = None
    testcase: Union[Text, Callable, None] = None
    variables: VariablesMapping = {}
    setup_hooks: Hooks = []
    teardown_hooks: Hooks = []
    # used to extract request's response field
    extract: VariablesMapping = {}
    # used to export session variables from referenced testcase
    export: Export = []
    validators: Validators = Field([], alias="validate")
    validate_script: List[Text] = []
    skip: Union[bool, Text] = False
    step_type: StepTypeEnum = Field(StepTypeEnum.UNKNOWN, description="步骤类型 api ui", alias="type")
    location: Union[List[TUiLocation], None] = None
    # 前置sql
    pre_sql: Union[List[SqlData], None] = None
    # 后置sql
    post_sql: Union[List[SqlData], None] = None


class TestCase(BaseModel):
    config: TConfig
    teststeps: List[TStep]


class ProjectMeta(BaseModel):
    debugtalk_py: Text = ""  # debugtalk.py file content
    debugtalk_path: Text = ""  # debugtalk.py file path
    dot_env_path: Text = ""  # .env file path
    functions: FunctionsMapping = {}  # functions defined in debugtalk.py
    env: Env = {}
    RootDir: Text = os.getcwd()  # project root directory (ensure absolute), the path debugtalk.py located


class TestsMapping(BaseModel):
    project_meta: ProjectMeta
    testcases: List[TestCase]


class TestCaseTime(BaseModel):
    """用例时间"""
    start_at: float = 0
    start_at_iso_format: Text = ""  # 开始系统时间
    duration: float = 0


class TestCaseInOut(BaseModel):
    """用例输入输出"""
    config_vars: VariablesMapping = {}
    export_vars: Dict = {}


class RequestStat(BaseModel):
    """请求统计"""
    content_size: float = 0  # 响应内容大小
    response_time_ms: float = 0  # 响应时间 毫秒
    elapsed_ms: float = 0  # 过程时间


class AddressData(BaseModel):
    """地址信息"""
    client_ip: Text = "N/A"
    client_port: int = 0
    server_ip: Text = "N/A"
    server_port: int = 0


class RequestData(BaseModel):
    method: MethodEnum = MethodEnum.GET
    url: Url
    headers: Headers = {}
    cookies: Cookies = {}
    body: Union[Text, bytes, List, Dict, None] = {}


class ResponseData(BaseModel):
    status_code: int
    headers: Dict
    cookies: Cookies
    encoding: Union[Text, None] = None
    content_type: Text
    body: Union[Text, bytes, List, Dict, None]


class ReqRespData(BaseModel):
    request: RequestData
    response: ResponseData


class SessionData(BaseModel):
    """ 请求会话数据 request session data, including request, response, validators and stat data"""

    success: bool = False
    # in most cases, req_resps only contains one request & response
    # while when 30X redirect occurs, req_resps will contain multiple request & response
    req_resps: List[ReqRespData] = []
    stat: RequestStat = RequestStat()
    address: AddressData = AddressData()
    validators: Dict = {}


class StepData(BaseModel):
    """teststep data, each step maybe corresponding to one request or one testcase"""

    success: bool = False
    name: Text = ""  # teststep name
    data: Union[SessionData, List['StepData']] = None
    export_vars: VariablesMapping = {}


StepData.update_forward_refs()


class TestCaseSummary(BaseModel):
    """用例汇总数据"""
    name: Text
    success: bool
    case_id: Text
    time: TestCaseTime
    in_out: TestCaseInOut = {}
    log: Text = ""
    step_datas: List[StepData] = []
    # ------------------- 20241029
    # run_count: int  # 运行数量
    # actual_run_count: int  # 实际执行数量
    # run_success_count: int  # 运行成功数
    # run_fail_count: int  # 运行错误数
    # run_skip_count: int  # 运行跳过数
    # run_err_count: int  # 运行错误数
    # start_time_iso_format: str  # 运行时间系统时间
    # # message 记录错误信息
    # message: str


class PlatformInfo(BaseModel):
    pastor_version: Text
    python_version: Text
    platform: Text


class TestCaseRef(BaseModel):
    name: Text
    base_url: Text = ""
    testcase: Text
    variables: VariablesMapping = {}


class TestSuite(BaseModel):
    config: TConfig
    testcases: List[TestCaseRef]


class Stat(BaseModel):
    """统计数据"""
    total: int = 0
    success: int = 0
    fail: int = 0


class TestSuiteSummary(BaseModel):
    """测试套件汇总数据"""
    success: bool = False
    stat: Stat = Stat()
    time: TestCaseTime = TestCaseTime()
    platform: PlatformInfo
    testcases: List[TestCaseSummary]



# NOTE: Generated By Pastor v3.1.7
# FROM: load_image.yml


from pastor import Pastor, Config, Step, RunRequest, RunTestCase


class TestCaseLoadImage(Pastor):

    config = Config("load images").base_url("${get_httpbin_server()}")

    teststeps = [
        Step(
            RunRequest("get png image")
            .get("/image/png")
            .validate()
            .assert_equal("status_code", 200)
        ),
        Step(
            RunRequest("get jpeg image")
            .get("/image/jpeg")
            .validate()
            .assert_equal("status_code", 200)
        ),
        Step(
            RunRequest("get webp image")
            .get("/image/webp")
            .validate()
            .assert_equal("status_code", 200)
        ),
        Step(
            RunRequest("get svg image")
            .get("/image/svg")
            .validate()
            .assert_equal("status_code", 200)
        ),
    ]


if __name__ == "__main__":
    TestCaseLoadImage().test_start()



# NOTE: Generated By Pastor v3.1.7
# FROM: validate.yml


from pastor import Pastor, Config, Step, RunRequest, RunTestCase


class TestCaseValidate(Pastor):

    config = Config("basic test with httpbin").base_url("http://httpbin.org/")

    teststeps = [
        Step(
            RunRequest("validate response with json path")
            .get("/get")
            .with_params(**{"a": 1, "b": 2})
            .validate()
            .assert_equal("status_code", 200)
            .assert_equal("body.args.a", "1")
            .assert_equal("body.args.b", "2")
        ),
        Step(
            RunRequest("validate response with python script")
            .get("/get")
            .with_params(**{"a": 1, "b": 2})
            .validate()
            .assert_equal("status_code", 200)
        ),
    ]


if __name__ == "__main__":
    TestCaseValidate().test_start()



# NOTE: Generated By Pastor v3.1.7
# FROM: cookie_manipulation/set_delete_cookies.yml


from pastor import Pastor, Config, Step, RunRequest, RunTestCase


class TestCaseSetDeleteCookies(Pastor):

    config = (
        Config("set & delete cookies.")
        .variables(**{"foo1": "bar1", "foo2": "bar2"})
        .base_url("https://postman-echo.com")
        .verify(False)
        .export(*["cookie_foo1", "cookie_foo3"])
    )

    teststeps = [
        Step(
            RunRequest("set cookie foo1 & foo2 & foo3")
            .with_variables(**{"foo3": "bar3"})
            .get("/cookies/set")
            .with_params(**{"foo1": "bar111", "foo2": "$foo2", "foo3": "$foo3"})
            .with_headers(**{"User-Agent": "Pastor/${get_pastor_version()}"})
            .extract()
            .with_jmespath("$.cookies.foo1", "cookie_foo1")
            .with_jmespath("$.cookies.foo3", "cookie_foo3")
            .validate()
            .assert_equal("status_code", 200)
            .assert_not_equal("$.cookies.foo3", "$foo3")
        ),
        Step(
            RunRequest("delete cookie foo2")
            .get("/cookies/delete?foo2")
            .with_headers(**{"User-Agent": "Pastor/${get_pastor_version()}"})
            .validate()
            .assert_equal("status_code", 200)
            .assert_not_equal("$.cookies.foo1", "$foo1")
            .assert_equal("$.cookies.foo1", "$cookie_foo1")
            .assert_equal("$.cookies.foo3", "$cookie_foo3")
        ),
    ]


if __name__ == "__main__":
    TestCaseSetDeleteCookies().test_start()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @time   :2024/10/29 16:18
# @Author : liangchunhua
# @Desc   :
import json
import os
from enum import Enum
from venv import logger

from pydantic import Field, BaseModel
from pydantic.v1 import HttpUrl
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from webdriver_manager_zh.chrome import ChromeDriverManager

from pastor.exceptions import ParamsError
from pastor.uicore.config import get_domain, is_read_cache
import validators

class BrowserTypeEnum(str, Enum):
    """浏览器类型"""
    CHROME = "Chrome"
    FIREFOX = "Firefox"
    EDGE = "Edge"
    IE = "IE"


class DriverSetting(BaseModel):
    """driver设置"""
    remote_url: str = Field("http://127.0.0.1:4444/wb/hub", description="远程地址")
    browser: str = Field(BrowserTypeEnum.CHROME, description="浏览器名称")
    headless: bool = Field(True, description="是否无头模式")
    page_load_timeout: int = Field(90, description="页面加载超时时间")
    implicitly_wait_time: int = Field(5, description="元素等待超时时间")


class AutoDriver():
    def __init__(self, ignore_browser_cache: bool = False):
        from pastor.uicore.config import get_remote_url, is_headless, get_page_load_timeout, \
            get_implicitly_wait_time, get_window_size
        setting = DriverSetting(
            remote_url=get_remote_url(),
            browser=BrowserTypeEnum.CHROME,
            headless=is_headless(),
            page_load_timeout=get_page_load_timeout(),
            implicitly_wait_time=get_implicitly_wait_time(),
        )
        self.page_load_timeout = setting.page_load_timeout
        self.implicitly_wait_time = setting.implicitly_wait_time
        if setting.browser == BrowserTypeEnum.CHROME:
            options = Options()
            # 设置浏览器尺寸
            options.add_argument(get_window_size())
            # 通过端口号接管已打开的浏览器
            # options.add_experimental_option("debuggerAddress", "127.0.0.1:9222")
            # 添加浏览器缓存
            user_directory_path = os.path.join(os.path.expanduser('~'), "selenium_chrome_cookies")
            if not os.path.exists(user_directory_path):
                # 缓存路径不存在时，直接创建
                options.add_argument(f'--user-data-dir={user_directory_path}')
            # options.add_argument(f'--disk-cache-dir={user_directory_path}')
            elif is_read_cache() == 'true' and ignore_browser_cache is False:
                options.add_argument(f'--user-data-dir={user_directory_path}')

            options.add_argument('--disable-gpu')
            # linux无头模式以root用户运行，开启可能会报错
            options.add_argument("--no-sandbox")
            # options.add_argument('--ignore-certificate-errors')
            # options.add_experimental_option("excludeSwitches",
            #                                 ['load-extension', 'enable-automation', 'enable-logging'])
            if setting.remote_url != 'http://127.0.0.1:4444/wb/hub':
                """远程执行"""
                # 默认vnc密码：secret
                if setting.headless:
                    options.add_argument("--headless")
                self.driver = webdriver.Remote(command_executor=setting.remote_url, options=options)
            else:
                """本地执行"""
                # os.environ['GH_TOKEN'] = '' # gitlab访问token
                from pastor.uicore.config import driver_latest_release_url, driver_download_url
                latest_release_url = driver_latest_release_url() if driver_latest_release_url() else 'https://googlechromelabs.github.io/chrome-for-testing/latest-patch-versions-per-build.json'
                url = driver_download_url() if driver_download_url() else 'https://googlechromelabs.github.io/chrome-for-testing/known-good-versions-with-downloads.json'
                # drivers = webdriver.Chrome(service=ChromeService(
                #     ChromeDriverManager(url=url, latest_release_url=latest_release_url).install()))
                # self.drivers = webdriver.Chrome(executable_path=setting.executable_path, options=options)
                driver_path = ChromeDriverManager(url=url, latest_release_url=latest_release_url).install()
                self.driver = webdriver.Chrome(options, service=webdriver.ChromeService(driver_path))
        else:
            raise Exception(f"暂不支持其他浏览器: {setting.browser}")
        # 元素等待超时时间
        self.driver.implicitly_wait(setting.implicitly_wait_time)  # seconds
        # 页面刷新超时时间
        self.driver.set_page_load_timeout(setting.page_load_timeout)  # seconds

    def open(self, url):
        r = validators.url(url)
        if not r:
            if self.driver:
                self.driver.quit()
            raise r
        self.driver.get(url)
        # if is_read_cookie() is True:
        #     self.read_browser_cookies(url)
        return self.driver

    def get_session_variables(self):
        """获取session变量"""
        return self._session_variables

    def quit(self):
        """退出"""
        self.driver.quit()

    def get_driver_session_id(self):
        """获取session_id"""
        return self.driver.session_id

    def get_screenshot(self, screenshot_type="base64", file_path=None):
        """截图"""
        if screenshot_type == "base64":
            """方法得到图片的base64编码"""
            return self.driver.get_screenshot_as_base64()
        elif screenshot_type == "png":
            """方法得到图片的二进制数据"""
            return self.driver.get_screenshot_as_png()
        elif screenshot_type == "file":
            """方法得到图片的二进制数据"""
            if not file_path:
                raise Exception("截图路径不能为空")
            return self.driver.get_screenshot_as_file(file_path)
        else:
            raise Exception(f"不支持的截图类型: {screenshot_type}")

    def save_browser_cookies(self):
        cookies = self.driver.get_cookies()
        json_cookies = json.dumps(cookies)
        directory = os.path.join(os.path.expanduser('~'), "selenium_chrome_cookies.json")
        logger.info(f'browser cookies save : {directory}')
        with open(directory, 'w') as f:
            f.write(json_cookies)

    def read_browser_cookies(self, url):
        self.driver.delete_all_cookies()
        directory = os.path.join(os.path.expanduser('~'), "selenium_chrome_cookies.json")
        with open(directory, 'r', encoding='utf-8') as f:
            cookie_list = json.loads(f.read())
        if len(cookie_list) > 0:
            for cookie in cookie_list:
                # 解决统一登录平台和实际登录域名不一致
                cookie['domain'] = get_domain()
                # 删除过期时间
                del cookie['expiry']
                self.driver.add_cookie(cookie)
            self.driver.get(url)
            self.driver.refresh()


def browser_type(bw_type, open_url):
    if bw_type == "Chrome":
        pass

    elif bw_type == "Edge":
        from selenium.webdriver.edge.service import Service as EdgeService
        from webdriver_manager_zh.microsoft import EdgeChromiumDriverManager
        driver = webdriver.Edge(service=EdgeService(EdgeChromiumDriverManager().install()))
        driver.get(open_url)

    elif bw_type == "Firefox":
        from selenium.webdriver.firefox.service import Service as FirefoxService
        from webdriver_manager_zh.firefox import GeckoDriverManager
        driver = webdriver.Firefox(service=FirefoxService(GeckoDriverManager().install()))
        driver.get(open_url)

    elif bw_type == "IE":
        from selenium.webdriver.ie.service import Service as IEService
        from webdriver_manager_zh.microsoft import IEDriverManager
        driver = webdriver.Ie(service=IEService(IEDriverManager().install()))
        driver.get(open_url)


def test():
    from webdriver_manager_zh.chrome import ChromeDriverManager
    from webdriver_manager_zh.core.os_manager import OperationSystemManager
    from webdriver_manager_zh.core.file_manager import FileManager
    from webdriver_manager_zh.core.driver_cache import DriverCacheManager

    # https://registry.npmmirror.com/-/binary/chrome-for-testing

    # 配置操作系统管理器
    os_manager = OperationSystemManager(os_type="win64")

    # 配置文件管理器
    file_manager = FileManager(os_system_manager=os_manager)

    # 配置驱动缓存管理器
    cache_manager = DriverCacheManager(file_manager=file_manager)

    # 配置 Chrome 驱动管理器
    chrome_manager = ChromeDriverManager(cache_manager=cache_manager)
    chrome_manager.install()

    # OperationSystemManager: 操作系统管理器，用于配置操作系统的类型。
    # FileManager: 文件管理器，用于管理文件操作。
    # DriverCacheManager: 驱动缓存管理器，用于管理驱动的缓存。
    # ChromeDriverManager: Chrome


if __name__ == '__main__':
    a = AutoDriver().open('https://www.baidu.com/')



__version__ = "1.0.1"
__description__ = "One-stop solution for AUTO testing."

# import firstly for monkey patch if needed
from pastor.ext.locust import main_locusts
from pastor.parser import parse_parameters as Parameters
from pastor.runner import Pastor
from pastor.testcase import Config, Step, RunRequest, RunTestCase, RunLocation

__all__ = [
    "__version__",
    "__description__",
    "Pastor",
    "Config",
    "Step",
    "RunLocation",
    "RunRequest",
    "RunTestCase",
    "Parameters",
    "Parameters2",
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @time   :2024/10/31 16:50
# @Author : liangchunhua
# @Desc   :
import validators

def open(url):
    r = validators.url(url)
    if not r:
        raise r
    else:
        print(2)
    print(3)

if __name__ == '__main__':
    open('1')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @time   :2024/10/30 10:18
# @Author : liangchunhua
# @Desc   : 等待元素，并返回元素类
from time import sleep

from selenium.common import TimeoutException
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

from pastor.models import TUiLocation
from pastor.uicore.driver import AutoDriver


def locator(auto_driver: AutoDriver, step: TUiLocation):
    wait = WebDriverWait(auto_driver.driver, auto_driver.implicitly_wait_time)
    if step.by in ('title', 'url', 'current_url'):
        return None
    else:
        try:
            location = wait.until(EC.presence_of_element_located(
                (getattr(By, step.by), step.value)))
        except:
            sleep(5)
            try:
                location = wait.until(EC.presence_of_element_located(
                    (getattr(By, step.by), step.value)))
            except TimeoutException:
                raise TimeoutException(f'定位元素: {step.by}({step.value}) 失败: 超时')
    try:
        if auto_driver.driver.name in ('chrome', 'safari'):
            auto_driver.driver.execute_script("arguments[0].scrollIntoViewIfNeeded(true)", location)
        else:
            auto_driver.driver.execute_script("arguments[0].scrollIntoView(false)", location)
    except:
        pass

    try:
        if step.action.upper() == 'CLICK':
            location = wait.until(EC.element_to_be_clickable(
                (getattr(By, step.by), step.value)))
        else:
            location = wait.until(EC.visibility_of_element_located(
                (getattr(By, step.by), step.value)))
    except:
        pass

    return location



from pastor import __version__


def get_pastor_version():
    return __version__


def sum_two(m, n):
    return m + n


def get_testcase_config_variables():
    return {"foo1": "testcase_config_bar1", "foo2": "testcase_config_bar2"}


def get_testsuite_config_variables():
    return {"foo1": "testsuite_config_bar1", "foo2": "testsuite_config_bar2"}


def get_app_version():
    return [3.1, 3.0]


def calculate_two_nums(a, b=1):
    return [a + b, b - a]



# NOTE: Generated By Pastor v3.1.9
# FROM: request_methods/hardcode.yml


from pastor import Pastor, Config, Step, RunRequest, RunTestCase


class TestCaseHardcode(Pastor):

    config = (
        Config("request methods testcase in hardcode")
        .base_url("https://postman-echo.com")
        .verify(False)
    )

    teststeps = [
        Step(
            RunRequest("get with params")
            .get("/get")
            .with_params(**{"foo1": "bar1", "foo2": "bar2"})
            .with_headers(**{"User-Agent": "Pastor/3.0"})
            .validate()
            .assert_equal("status_code", 200)
        ),
        Step(
            RunRequest("post raw text")
            .post("/post")
            .with_headers(
                **{"User-Agent": "Pastor/3.0", "Content-Type": "text/plain"}
            )
            .with_data("This is expected to be sent back as part of response body.")
            .validate()
            .assert_equal("status_code", 200)
        ),
        Step(
            RunRequest("post form data")
            .post("/post")
            .with_headers(
                **{
                    "User-Agent": "Pastor/3.0",
                    "Content-Type": "application/x-www-form-urlencoded",
                }
            )
            .with_data("foo1=bar1&foo2=bar2")
            .validate()
            .assert_equal("status_code", 200)
        ),
        Step(
            RunRequest("put request")
            .put("/put")
            .with_headers(
                **{"User-Agent": "Pastor/3.0", "Content-Type": "text/plain"}
            )
            .with_data("This is expected to be sent back as part of response body.")
            .validate()
            .assert_equal("status_code", 200)
        ),
    ]


if __name__ == "__main__":
    TestCaseHardcode().test_start()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @time   :2024/10/30 16:18
# @Author : liangchunhua
# @Desc   : 页面元素操作

from time import sleep
from typing import List

from loguru import logger
from selenium.common.exceptions import ElementClickInterceptedException, TimeoutException
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait

from pastor.models import TUiLocation
from pastor.uicore.driver import AutoDriver
from pastor.uicore.locator import locator


class Action:

    @staticmethod
    def open(auto_driver: AutoDriver, step: TUiLocation):
        driver = auto_driver.open(step.data)
        if step.cookie:
            driver.add_cookie(step.cookie)
        sleep(0.5)

    @staticmethod
    def title(auto_driver: AutoDriver):
        """获取页面标题"""
        return auto_driver.driver.title

    @staticmethod
    def current_url(auto_driver: AutoDriver):
        """获取当前页面的url"""
        return auto_driver.driver.current_url

    @staticmethod
    def check(step: TUiLocation, auto_driver: AutoDriver):
        pass

    @staticmethod
    def notcheck(step):
        data = step['data']
        if not data:
            data = step['expected']
        element = step['element']
        # element_location = locator_element(element)

        # if element['by'] == 'title':
        #     assert data['text'] != g.driver.title

    @staticmethod
    def send_keys(auto_driver: AutoDriver, step: TUiLocation):
        element_location = locator(auto_driver, step)
        ActionChains(auto_driver.driver).double_click(element_location).perform()
        element_location.send_keys(step.data)
        return element_location

    @staticmethod
    def switch_to_frame(auto_driver: AutoDriver, step: TUiLocation):
        element_location = locator(auto_driver, step)
        # auto_driver.driver.implicitly_wait(5)
        auto_driver.driver.switch_to.frame(element_location)

    @staticmethod
    def click(auto_driver: AutoDriver, step: TUiLocation):
        element_location = locator(auto_driver, step)
        if element_location:
            try:
                element_location.click()
            except ElementClickInterceptedException:  # 如果元素为不可点击状态，则等待1秒，再重试一次
                sleep(1)
                element_location.click()
        sleep(0.5)
        return element_location

    @staticmethod
    def context_click(auto_driver: AutoDriver, step: TUiLocation):
        """
        右击
        :param auto_driver:
        :param step:
        :return:
        """
        actions = ActionChains(auto_driver.driver)
        element_location = locator(auto_driver, step)
        actions.context_click(element_location)
        actions.perform()
        sleep(0.5)
        return element_location

    @staticmethod
    def finds(auto_driver: AutoDriver, step: TUiLocation) -> List[WebElement]:
        wait = WebDriverWait(auto_driver.driver, 2)
        try:
            return wait.until(EC.presence_of_all_elements_located(
                (getattr(By, step.by), step.value)))
        except TimeoutException as e:
            logger.info(f'{step.by}({step.value})调用finds查询超时，返回空数组')
            return []

    @staticmethod
    def find_descendant(auto_driver: AutoDriver, step: TUiLocation):
        """
        查找当前节点的所有后代元素（子、孙等）,不包含当前节点
        :param auto_driver:
        :param step:
        :return:
        """
        element_location = locator(auto_driver, step)
        return element_location.find_elements(By.XPATH, './descendant::*')

    @staticmethod
    def find_descendant_or_self(auto_driver: AutoDriver, step: TUiLocation):
        """
        查找当前节点的所有后代元素（子、孙等）,包含当前节点
        :param auto_driver:
        :param step:
        :return:
        """
        element_location = locator(auto_driver, step)
        return element_location.find_elements(By.XPATH, './descendant-or-self::*')

    @staticmethod
    def find(auto_driver: AutoDriver, step: TUiLocation):
        return Action.finds(auto_driver, step)[0]

    @staticmethod
    def hover(auto_driver: AutoDriver, step: TUiLocation):
        actions = ActionChains(auto_driver.driver)
        element_location = locator(auto_driver, step)
        actions.move_to_element(element_location)
        actions.perform()
        sleep(0.5)
        return element_location

    @staticmethod
    def double_click(auto_driver: AutoDriver, step: TUiLocation):
        actions = ActionChains(auto_driver.driver)
        element_location = locator(auto_driver, step)
        actions.double_click(element_location)
        actions.perform()
        sleep(0.5)
        return element_location

    @staticmethod
    def drag_and_drop(auto_driver: AutoDriver, step: TUiLocation):
        data = step.data
        """拖拽"""
        actions = ActionChains(auto_driver.driver)
        source = locator(auto_driver, data[0])
        target = locator(auto_driver, data[1])
        actions.drag_and_drop(source, target)
        actions.perform()
        sleep(0.5)

    @staticmethod
    def swipe(auto_driver: AutoDriver, step: TUiLocation):
        """滑动"""
        data = step.data
        actions = ActionChains(auto_driver.driver)
        source = locator(auto_driver, step)
        x = data.get('x', 0)
        y = data.get('y', 0)
        actions.drag_and_drop_by_offset(source, x, y)
        actions.perform()
        sleep(0.5)

    @staticmethod
    def execute_script(auto_driver: AutoDriver, step: TUiLocation):
        """
        执行脚本
        :param auto_driver:
        :param step:
        :return:
        """
        return auto_driver.driver.execute_script(step.data)

    @staticmethod
    def message(auto_driver: AutoDriver, step: TUiLocation):
        data = step['data']
        text = data.get('text', '')
        element = step['element']
        value = ''  # e.get(element)[1]

        # if value.lower() in ('确认', 'accept'):
        #     g.driver.switch_to_alert().accept()
        # elif value.lower() in ('取消', '关闭', 'cancel', 'close'):
        #     g.driver.switch_to_alert().dismiss()
        # elif value.lower() in ('输入', 'input'):
        #     g.driver.switch_to_alert().send_keys(text)
        #     g.driver.switch_to_alert().accept()
        logger.info('--- Switch Frame: Alert')
        # w.frame = 'Alert'

    @staticmethod
    def upload(auto_driver: AutoDriver, step: TUiLocation):
        """上传文件"""
        import win32com.client

        data = step['data']
        element = step['element']
        element_location = locator(auto_driver, step)
        file_path = data.get('text', '') or data.get('file', '')

        element_location.click()
        sleep(3)
        shell = win32com.client.Dispatch("WScript.Shell")
        shell.Sendkeys(file_path)
        sleep(2)
        shell.Sendkeys("{ENTER}")
        sleep(2)

    @staticmethod
    def navigate(auto_driver: AutoDriver, step: TUiLocation):
        """导航"""
        if step.value.lower() in ('刷新', 'refresh'):
            auto_driver.driver.refresh()
        elif step.value.lower() in ('前进', 'forward'):
            auto_driver.driver.forward()
        elif step.value.lower() in ('后退', 'back'):
            auto_driver.driver.back()

    @staticmethod
    def scroll(auto_driver: AutoDriver, step: TUiLocation):
        data = step.data
        """滚动"""
        x = data.get('x', 0)
        y = data.get('y', 0)

        element = step['element']
        if element == '':
            if y:
                auto_driver.driver.execute_script(f"document.documentElement.scrollTop={y}")
            if x:
                auto_driver.driver.execute_script(f"document.documentElement.scrollLeft={x}")
        else:
            element_location = locator(auto_driver, step)

            if y:
                auto_driver.driver.execute_script(f"arguments[0].scrollTop={y}", element_location)
            if x:
                auto_driver.driver.execute_script(f"arguments[0].scrollLeft={x}", element_location)

    @staticmethod
    def wait_page_loaded(auto_driver: AutoDriver, step: TUiLocation):
        ready_state = auto_driver.driver.execute_script("return document.readyState")
        if ready_state != 'complete':
            if not isinstance(step.data, int):
                step.data = 10
            WebDriverWait(auto_driver.driver, step.data).until(
                lambda driver: driver.execute_script("return document.readyState") == "complete")

    @staticmethod
    def text_to_be_present_in_element(auto_driver: AutoDriver, step: TUiLocation):
        """
        等待元素文本包含text
        :param auto_driver:
        :param step:
        :param data: 超时时间
        :return:
        """
        WebDriverWait(auto_driver.driver, auto_driver.implicitly_wait_time).until(EC.text_to_be_present_in_element(
            (getattr(By, step.by), step.value), step.data))

    @staticmethod
    def visibility_of_element_located(auto_driver: AutoDriver, step: TUiLocation):
        """
        等待某个元素是否可见. 可见代表元素非隐藏，并且元素的宽和高都不等于0
        :param auto_driver:
        :param step:
        :param data: 超时时间
        :return:
        """
        WebDriverWait(auto_driver.driver, step.data).until(EC.visibility_of_element_located(
            (getattr(By, step.by), step.value)))

    @staticmethod
    def element_to_be_clickable(auto_driver: AutoDriver, step: TUiLocation):
        """
        等待可点击状态
        :param auto_driver:
        :param step:
        :return:
        """
        WebDriverWait(auto_driver.driver, step.data).until(EC.element_to_be_clickable(
            (getattr(By, step.by), step.value)))

    @staticmethod
    def presence_of_element_located(auto_driver: AutoDriver, step: TUiLocation):
        """
        等待元素加载到dom树里，并不代表该元素一定可见
        :param auto_driver:
        :param step:
        :return:
        """
        WebDriverWait(auto_driver.driver, step.data).until(EC.presence_of_element_located(
            (getattr(By, step.by), step.value)))

    @staticmethod
    def staleness_of(auto_driver: AutoDriver, step: TUiLocation):
        """
        等待某个元素从DOM中消失
        :param auto_driver:
        :param step:
        :return:
        """
        ele = auto_driver.driver.find_element(getattr(By, step.by), step.value)
        WebDriverWait(auto_driver.driver, step.data).until(EC.staleness_of(ele))





from setuptools import find_packages, setup


setup(
    name="pastor",
    version="1.0.2",
    python_requires=">=3.7",
    packages=find_packages(exclude=["examples", "tests", "tests.*"]),
    description="ui and api autotest",
    author="chliang",
    install_requires = [
        'selenium~=4.25.0',
        'requests~=2.32.3',
        'webdriver_manager_zh~=4.0.3',
        'pytest~=8.3.3',
        'urllib3~=2.2.3',
        'loguru~=0.7.2',
        'PyYAML~=6.0.2',
        'pydantic~=2.9.2',
        'jinja2~=3.1.4',
        'sentry_sdk==2.17.0',
        'jmespath~=1.0.1',
        'starlette~=0.41.2',
        'fastapi~=0.115.4',
        'SQLAlchemy~=2.0.36',
        'requests_toolbelt~=1.0.0',
        'pywin32==308',
        'wheel==0.44.0',
        'pymysql==1.1.1',
        'black==24.10.0',
        'validators==0.34.0'
    ],
    entry_points={
        'console_scripts': [
            'har2case=pastor.cli:main_har2case_alias',
            'pmake=pastor.cli:main_make_alias',
            'prun=pastor.cli:main_prun_alias',
            'pastor=pastor.cli:main',
            'locusts=pastor.ext.locust:main_locusts'
        ]
    }
)


# NOTE: Generated By Pastor v3.1.7
# FROM: upload.yml


from pastor import Pastor, Config, Step, RunRequest, RunTestCase


class TestCaseUpload(Pastor):

    config = Config("test upload file with httpbin").base_url("${get_httpbin_server()}")

    teststeps = [
        Step(
            RunRequest("upload file")
            .with_variables(
                **{
                    "file_path": "test.env",
                    "m_encoder": "${multipart_encoder(file=$file_path)}",
                }
            )
            .post("/post")
            .with_headers(**{"Content-Type": "${multipart_content_type($m_encoder)}"})
            .with_data("$m_encoder")
            .validate()
            .assert_equal("status_code", 200)
            .assert_startswith("body.files.file", "UserName=test")
        ),
        Step(
            RunRequest("upload file with keyword")
            .post("/post")
            .upload(**{"file": "test.env"})
            .validate()
            .assert_equal("status_code", 200)
            .assert_startswith("body.files.file", "UserName=test")
        ),
    ]


if __name__ == "__main__":
    TestCaseUpload().test_start()



# NOTE: Generated By Pastor v3.1.7
# FROM: basic.yml


from pastor import Pastor, Config, Step, RunRequest, RunTestCase


class TestCaseBasic(Pastor):

    config = Config("basic test with httpbin").base_url("https://httpbin.org/")

    teststeps = [
        Step(
            RunRequest("headers")
            .get("/headers")
            .validate()
            .assert_equal("status_code", 200)
            .assert_equal("body.headers.Host", "httpbin.org")
        ),
        Step(
            RunRequest("user-agent")
            .get("/user-agent")
            .validate()
            .assert_equal("status_code", 200)
            .assert_startswith('body."user-agent"', "python-requests")
        ),
        Step(
            RunRequest("get without params")
            .get("/get")
            .validate()
            .assert_equal("status_code", 200)
            .assert_equal("body.args", {})
        ),
        Step(
            RunRequest("get with params in url")
            .get("/get?a=1&b=2")
            .validate()
            .assert_equal("status_code", 200)
            .assert_equal("body.args", {"a": "1", "b": "2"})
        ),
        Step(
            RunRequest("get with params in params field")
            .get("/get")
            .with_params(**{"a": 1, "b": 2})
            .validate()
            .assert_equal("status_code", 200)
            .assert_equal("body.args", {"a": "1", "b": "2"})
        ),
        Step(
            RunRequest("set cookie")
            .get("/cookies/set?name=value")
            .validate()
            .assert_equal("status_code", 200)
            .assert_equal("body.cookies.name", "value")
        ),
        Step(
            RunRequest("extract cookie")
            .get("/cookies")
            .validate()
            .assert_equal("status_code", 200)
            .assert_equal("body.cookies.name", "value")
        ),
        Step(
            RunRequest("post data")
            .post("/post")
            .with_headers(**{"Content-Type": "application/json"})
            .with_data("abc")
            .validate()
            .assert_equal("status_code", 200)
        ),
        Step(
            RunRequest("validate body length")
            .get("/spec.json")
            .validate()
            .assert_length_equal("body", 9)
        ),
    ]


if __name__ == "__main__":
    TestCaseBasic().test_start()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @time   :2024/11/1 13:32
# @Author : liangchunhua
# @Desc   : ui自动化元素结果校验
import time
from typing import Dict, Text, Any, List

import jmespath
import requests
from jmespath.exceptions import JMESPathError
from loguru import logger
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webelement import WebElement

from pastor import exceptions
from pastor.exceptions import ValidationFailure, ParamsError, FunctionNotFound
from pastor.models import VariablesMapping, Validators, FunctionsMapping, TUiLocation
from pastor.parser import parse_data, parse_string_value, get_mapping_function
from pastor.uicore.driver import AutoDriver
from pastor.uicore.web_action import Action


def get_uniform_comparator(comparator: Text):
    """
    convert comparator alias to uniform name
    """
    if comparator in ["eq", "equals", "equal"]:
        return "equal"
    elif comparator in ["lt", "less_than"]:
        return "less_than"
    elif comparator in ["le", "less_or_equals"]:
        return "less_or_equals"
    elif comparator in ["gt", "greater_than"]:
        return "greater_than"
    elif comparator in ["ge", "greater_or_equals"]:
        return "greater_or_equals"
    elif comparator in ["ne", "not_equal"]:
        return "not_equal"
    elif comparator in ["str_eq", "string_equals"]:
        return "string_equals"
    elif comparator in ["len_eq", "length_equal"]:
        return "length_equal"
    elif comparator in [
        "len_gt",
        "length_greater_than",
    ]:
        return "length_greater_than"
    elif comparator in [
        "len_ge",
        "length_greater_or_equals",
    ]:
        return "length_greater_or_equals"
    elif comparator in ["len_lt", "length_less_than"]:
        return "length_less_than"
    elif comparator in [
        "len_le",
        "length_less_or_equals",
    ]:
        return "length_less_or_equals"
    elif comparator in [
        "cont",
        "contains",
    ]:
        # 包含 expect_value
        return "contains"
    elif comparator in [
        "cont_by",
        "contained_by",
    ]:
        # expect_value 包含check_value
        return "contained_by"
    elif comparator in [
        "startswith",
    ]:
        return "startswith"
    elif comparator in [
        "endswith",
    ]:
        return "endswith"
    elif comparator in [
        "regex_match",
    ]:
        return "regex_match"
    else:
        return comparator


class ElementObj:

    def __init__(self, driver: AutoDriver):
        self.__check_item = None
        self.__assert_method = None
        self.driver = driver
        self.validation_results: Dict = {}

    def uniform_validator_element(self, validator, variables_mapping: VariablesMapping):
        if not isinstance(validator, dict):
            raise ParamsError(f"invalid validator: {validator}")

        if len(validator) == 1:
            # format2
            comparator = list(validator.keys())[0]
            compare_values = validator[comparator]

            if not isinstance(compare_values, dict) or len(compare_values) not in [1, 2, 3, 4, 5]:
                # 1, 2代表sql值校验， 3,4,5代表其他值校验
                raise ParamsError(f"invalid validator: {validator}")
            check_item, expect_value, elements = None, None, []
            if (len(compare_values) == 1 or len(compare_values) == 2) and next(iter(compare_values)) in variables_mapping:
                # 判断为检验sql字段值 , len=2 为message
                check_item = next(iter(compare_values))
                expect_value = compare_values.get(check_item)
            else:
                for key, value in compare_values.items():
                    if str(key).lower() in ['by', 'value', 'msg', 'action']:
                        continue
                    check_item = str(key)
                    expect_value = value

                # message = "" if 'msg' not in compare_values else compare_values['msg']

                ui_location_obj = TUiLocation.model_validate(compare_values)
                action = 'finds' if not ui_location_obj.action else ui_location_obj.action
                if not hasattr(Action, action):
                    funcs = [w for w in dir(Action) if callable(getattr(Action, w)) and not w.startswith("__")]
                    logger.error(f'action: {action}, 操作方法不存在以下列表中:{funcs}')
                    raise AttributeError(f'action: {action}, 操作方法不存在以下列表中:{funcs}')

                elements = getattr(Action, action)(self.driver, ui_location_obj)
                # logger.info(f'元素定位查询结果：{[vars(element) for element in elements]}')
        else:
            raise ParamsError(f"invalid validator: {validator}")

        message = "" if 'msg' not in compare_values else compare_values['msg']
        # uniform comparator, e.g. lt => less_than, eq => equals
        assert_method = get_uniform_comparator(comparator)
        self.__check_item = check_item
        return {
            "check": check_item,
            "expect": expect_value,
            "assert": assert_method,
            "message": message,
            "elements": elements,
            # make.py中使用
            "compare_values": compare_values,
        }

    def get_element_check_value(self, elements: List[WebElement], variables_mapping: VariablesMapping):
        if 'len' == self.__check_item:
            return len(elements)
        elif 'visible' == self.__check_item:
            # 元素是否存在,true存在，false不存在
            return False if len(elements) == 0 else True
        elif self.__check_item in variables_mapping:
            return variables_mapping[self.__check_item]
        else:
            if 'val' == str(self.__check_item).lower():
                if len(elements) == 0:
                    return None
                if len(elements) == 1:
                    # 查找当前元素及其所有后代元素的text
                    ele = elements[0]
                    texts = [s.strip() for s in ele.text.split('\n')]
                    return ','.join(texts)
                else:
                    # 数据验证, 获取List每个WebElement的text
                    texts = []
                    for element in elements:
                        # 标签文本内容
                        text = element.text.replace('\n', '').strip()
                        texts.append(text)
                    return ','.join(texts)
            if len(elements) > 0:
                ele = elements[0]
                # 获取属性值 class,id,style,title,src,href,alt,type,value,name,placeholder,disabled,checked,selected,
                # readonly(不会让元素变为灰色),rel,text
                attribute_value = ele.get_attribute(self.__check_item)
                if attribute_value:
                    return attribute_value
            return None

    def validate(
            self,
            validators: Validators,
            variables_mapping: VariablesMapping = None,
            functions_mapping: FunctionsMapping = None,
    ):

        variables_mapping = variables_mapping or {}
        functions_mapping = functions_mapping or {}

        self.validation_results = {}
        if not validators:
            return

        validate_pass = True
        failures = []

        for v in validators:

            if "validate_extractor" not in self.validation_results:
                self.validation_results["validate_extractor"] = []

            u_validator = self.uniform_validator_element(v, variables_mapping.setdefault('sql_return_data', {}))

            # check item
            check_item = u_validator["check"]
            elements = list(u_validator["elements"])

            check_value = self.get_element_check_value(elements, variables_mapping.setdefault('sql_return_data', {}))

            # comparator
            assert_method = u_validator["assert"]

            assert_func = get_mapping_function(assert_method, functions_mapping)

            # expect item
            expect_item = u_validator["expect"]
            # parse expected value with config/teststep/extracted variables
            expect_value = parse_data(expect_item, variables_mapping, functions_mapping)

            # message
            message = u_validator["message"]
            # parse message with config/teststep/extracted variables
            message = parse_data(message, variables_mapping, functions_mapping)

            validate_msg = f"assert {check_value} {assert_method} {expect_value}({type(expect_value).__name__})"
            validator_dict = {
                "comparator": assert_method,
                "check": check_item,
                "check_value": check_value,
                "expect": expect_item,
                "expect_value": expect_value,
                "message": message,
            }

            try:
                if str(assert_func.__name__).startswith('check_'):
                    # 自定义校验方法需要放在validate.py文件中，并且以check_开头
                    assert_func(elements, expect_value, message)
                else:
                    assert_func(check_value, expect_value, message)
                validate_msg += "\t==> pass"
                logger.info(validate_msg)
                validator_dict["check_result"] = "pass"
            except AssertionError as ex:
                validate_pass = False
                validator_dict["check_result"] = "fail"
                validate_msg += "\t==> fail"
                validate_msg += (
                    f"\n"
                    f"check_item: {check_item}\n"
                    f"check_value: {check_value}({type(check_value).__name__})\n"
                    f"assert_method: {assert_method}\n"
                    f"expect_value: {expect_value}({type(expect_value).__name__})\n"
                )
                if self.__check_item not in variables_mapping.setdefault('sql_return_data', {}):
                    validate_msg += f"query_elements_class: "
                    for items in v.values():
                        if isinstance(items, dict):
                            validate_msg += f" by={items['by']},value={items['value']}\n"
                    if len(elements) == 0:
                        validate_msg += '\t → elements is empty'
                    else:
                        for ele in elements:
                            validate_msg += f"\t → {ele.get_attribute('class')}\n"

                message = str(ex)
                if message:
                    validate_msg += f"\nmessage: {message}"

                logger.error(validate_msg)
                failures.append(validate_msg)

            self.validation_results["validate_extractor"].append(validator_dict)

        if not validate_pass:
            failures_string = "\n".join([failure for failure in failures])
            raise ValidationFailure(failures_string)



# NOTE: Generated By Pastor v3.1.9
# FROM: request_methods/request_with_testcase_reference.yml


import sys
from pathlib import Path

sys.path.insert(0, str(Path(__file__).parent.parent))


from pastor import Pastor, Config, Step, RunRequest, RunTestCase

from request_methods.request_with_functions_test import (
    TestCaseRequestWithFunctions as RequestWithFunctions,
)


class TestCaseRequestWithTestcaseReference(Pastor):

    config = (
        Config("request with referenced testcase")
        .variables(
            **{
                "foo1": "testcase_ref_bar12",
                "expect_foo1": "testcase_ref_bar12",
                "expect_foo2": "testcase_ref_bar22",
                "foo2": "testcase_ref_bar22",
            }
        )
        .base_url("https://postman-echo.com")
        .verify(False)
        .locust_weight(3)
    )

    teststeps = [
        Step(
            RunTestCase("request with functions")
            .with_variables(
                **{"foo1": "testcase_ref_bar1", "expect_foo1": "testcase_ref_bar1"}
            )
            .setup_hook("${sleep(0.1)}")
            .call(RequestWithFunctions)
            .teardown_hook("${sleep(0.2)}")
            .export(*["foo3"])
        ),
        Step(
            RunRequest("post form data")
            .with_variables(**{"foo1": "bar1"})
            .post("/post")
            .with_headers(
                **{
                    "User-Agent": "Pastor/${get_pastor_version()}",
                    "Content-Type": "application/x-www-form-urlencoded",
                }
            )
            .with_data("foo1=$foo1&foo2=$foo3")
            .validate()
            .assert_equal("status_code", 200)
            .assert_equal("body.form.foo1", "bar1")
            .assert_equal("body.form.foo2", "bar21")
        ),
    ]


if __name__ == "__main__":
    TestCaseRequestWithTestcaseReference().test_start()
